#!/usr/bin/perl -w
#
# pkt2html.pl - v0.1 
# This is a script to toss messages from inbound fidonet packets
# to text or html files.
# Copyright (c) 2001-2002 Robert James Clay & Russ Johnson.
# All Rights Reserved. This is free software;  you can redistribute
# it and/or modify it under the same terms as Perl itself.

# Setup
my $p2hvers = "v0.1";

use strict;
use File::Copy;
use Getopt::Std;
use vars qw/ $opt_i $opt_o $opt_l $opt_e $opt_t $opt_x $opt_h $opt_k /;

use FTN::Utility qw(&logging);

my $progid = "rpkt";

my ($attrib, $area, $datetime, $fromnode, $tonode, $from, $OUTDIR, $outfile, 
    $to, $subj, $msgbody, $msgid, $replyid, $logfile, $FILE,
    $cost, $osep, $orgnet, $destnet, $sec, $min, $hour, $mday, $mon, $year,
    $wday, $orgnode, $destnode, $seenby, $mailer, $origin, $buf, $s, $k, $i,
    $DEBUG, $PKTver, $TYPE, $PKT, $KEEPPKT, $INDIR, $FILENAME, $EXT,
    $next, $previous, %areas, $MsgTemp, $index, $dir, $echolist, $line,
    $arealist, $description, $thisarea, $newline, $last, 
    $kludges, $org, $dest, $pagespec, $section, $block, @l, @PKTS, @echolist,
    @dirs, @lines, @moy, @wdy, @time, @files, @args, @kludges, @dir);

my $TEMPDIR = "/tmp/pkt2html.$$";

# Archivers
# my $ZIP = "/usr/bin/zip";
# my $ARC = "/usr/bin/arc";

# Unarchiver
my $UNZIP = "/usr/bin/unzip";
my $UNARC = "/usr/bin/arc";

# No user serviceable parts inside

&commandline();

&checkformail();

&expandmail();

&tossmail();

&cleanup();

############################################
# subroutines
############################################

############################################
# Check for command line arguments, handle.
############################################
# Add handlers for other command line 
# arguments here
############################################
sub commandline {

    getopts('i:o:l:e:txhk');

    if ($opt_h) {
        &help ();
	exit(1);
    }

    if ($opt_x) {
        $DEBUG = $opt_x;	# set debug flag
        undef $opt_x;
        print "\nDebug flag is set ...\n";
    } else {
        $DEBUG = 0;
    }
	
    if ($opt_i) {
        $INDIR=$opt_i;  # this needs to be the filename & path
        undef $opt_i;
    } else {
        $INDIR="inbound";		# default inbound dir
    }

    if ($opt_o) {
        $OUTDIR=$opt_o;  # this needs to be the filename & path
        undef $opt_o;
    } else {
        $OUTDIR="msgs";		# default outbound dir
    }

    if ($opt_e) {
        # Echolist name and location
        @echolist = $opt_e;
#        @echolist = qw($opt_e);  #??
        undef $opt_e;
    } else {
        @echolist ="./backbone.na";	# default filename for echolistfile
    }
    print "\nEcholist info is from: @echolist\n";

    if ($opt_l) {
        $logfile=$opt_l;  # this needs to be the filename & path
        undef $opt_l;
    } else {
        $logfile="pkt2html.log";		# default log file
    }

    if ($opt_k) {
        $KEEPPKT = $opt_k;	# set keep packets flag
        undef $opt_k;
        print "\nKeeping FTN packets...\n";
    } else {
        $KEEPPKT = 0;
    }
	
    if ($opt_t) {
        print "\nContinuing...\n";
        return;
    }
    
    &help ();
    exit(1);
}

############################################
# Help
############################################
sub help {
    print "\nUsage:  pkt2html.pl -i InboundDir -o OutboundDir [-f text|html ] -t /";
    print "                     [-l logfilename] [-d] \n\n";
    print "   --toss  -t	Toss Mail\n";
    print "   --debug -x	Debug Mode\n";
    print "           -i   Inbound FTN archive/packet Dir (defaults to 'inbound')\n";
    print "           -k   keep incoming FTN packets (defaults to deleting them)\n";
    print "           -o   Outbound Dir (defaults to 'msgs')\n";
    print "   --log   -l   Log filename (defaults to pkt2html.log)\n";
    print "           -e   Echolist file & path\n";
    print "\n";
}

############################################
# Check for new mail
############################################
sub checkformail {
	opendir(DIR, $INDIR);
	@files = grep(/.*\.s[ua][0-9a-z]|mo[0-9a-z]|t[hu][0-9a-z]|we[0-9a-z]|fr[0-9a-z]|pkt/i, readdir(DIR));
	closedir(DIR);
	exit if ($#files == -1);
}

############################################
# If new mail, expand
############################################
sub expandmail {
	# Unarchive mail.
	
	if ($DEBUG) {print "unarchive\n"};	# for testing
	
	foreach $FILE (sort @files) {
		
		# if $FILE = "." or "..", do next file
		
		if ($DEBUG) {print "$INDIR/$FILE\n"};
		
		($FILENAME, $EXT) = split (/\./, "$FILE");
		
		undef $FILENAME;
				
		if ( "$EXT" ne "pkt" ) {
			open ARCHIVE, "$INDIR/$FILE" or die "Unable to open $FILE: $!\n";
			read ARCHIVE, $TYPE, 2;
				
			if ($TYPE eq "PK") {
				@args = ("$UNZIP", "-q", "$INDIR/$FILE", "-d $TEMPDIR");
			}
			else {
				# Default unarchiver
				@args = ("$UNARC", "x", "$INDIR/$FILE");
			}
			
			!system "@args" or die "@args failed: $?";
			
			close ARCHIVE;
		}
		else {
			if ( ! -d "$TEMPDIR" ) {
				mkdir ("$TEMPDIR", 0750 );
			}
			
			copy("$INDIR/$FILE", "$TEMPDIR/$FILE");
		}
		
		# delete packets if flag is not set
		if (!$KEEPPKT) {
		    unlink ("$INDIR/$FILE");
		}
	}
}

############################################
# Toss expanded packets
############################################
sub tossmail {
	# Toss mail
	
	if ($DEBUG) {print "Tossing...\n"};	# for testing
	
	opendir(TEMPDIR, $TEMPDIR);
#	@PKTS = readdir(TEMPDIR);
        @PKTS = grep(/.*\.pkt/i, readdir(TEMPDIR));  # packet files only
	closedir(TEMPDIR);
	
	foreach $PKT (sort @PKTS) {
		if ($DEBUG) {print "Toss messages here\n";
		print "$TEMPDIR/$PKT\n"};
		$PKT = "$TEMPDIR/$PKT";
		open(PKT,$PKT) || &logging($logfile, $progid, "$PKT,$!");
		binmode(PKT);
		
#		&log("Open: $PKT"); 
	        &logging($logfile, $progid, "Open: $PKT");
		
		&newmsgs();
		
#		&log("Deleting: $PKT"); 
	        &logging($logfile, $progid, "Deleting: $PKT");

		unlink($PKT);
	}
}

############################################
# Post New Messages
############################################
sub newmsgs 
{
#    local($to,$from,$subj); #,$msgid);
#    local($buf,$s,$osep); # $c Removed rlj


    read(PKT,$buf,58);  

    while (!eof(PKT))
    {
          last if (read(PKT,$buf,14) != 14);
          ($PKTver,$orgnode,$destnode,$orgnet,$destnet,$attrib,$cost) = unpack("SSSSSSS",$buf);

	  undef $PKTver;		#  not used for anything yet - 8/26/01 rjc
	  undef $attrib;		#  not used for anything yet - 8/26/01 rjc

          $osep = $/;                   
          $/ = "\0";                    

          $datetime = <PKT>;         
          if (length($datetime) > 20) 
          {
             $to = substr($datetime,20);
          } else {
             $to = <PKT>;
          }
          $from = <PKT>;
          $subj = <PKT>;

          $to       =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $to       =~ tr/\0-\037/\040-\100/;     # mask control characters
          $from     =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $from     =~ tr/\0-\037/\040-\100/;     # mask control characters
          $datetime =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $datetime =~ tr/\0-\037/\040-\100/;     # mask control characters
          $subj     =~ tr/\0-\037/\040-\100/;     # mask control characters

          $s = <PKT>;      
          $/ = $osep;

          $s =~ s/\x8d/\r/g;    
          @lines = split(/\r/,$s);  

          undef $s;     

          next if ($#lines < 0);      

          $area = shift(@lines);          
          $_=$area;
          $area="netmail" if /\//i; 
          $area =~ s/.*://;			# strip "area:"
          $area =~ tr/A-Z/a-z/;			# Lower case area
          
          # is this the first message in this area?
          
          if ( opendir(DIR, "$OUTDIR/$area") ) {
          	closedir(DIR);
          	findhighmsg();
          }
          else {
          	mkdir ("$OUTDIR/$area", 0755 );	
          	&makenewindex();
          	findhighmsg();
          }


          for ($i = $k = 0; $i <= $#lines; $i++) 
          {
               if ($lines[$i] =~ /^\001/) 
               {       
                  $kludges[$k++] = splice(@lines,$i,1);
                  redo;
               }
          }
          
    for (;;) {
              $_ = pop(@lines);
              last if ($_ eq "");  
          if (/ \* origin: /i) 
          {
             $origin = substr($_,11);
             last;
          }
          if (/---/) 
          {
             $mailer = $_;
          }
          if (/seen-by/i) 
          {
            $seenby=$_;
          }
    }
      
      
    if ($#lines < 0)           
    {
       @lines = ("[empty message]");
    }
    
    $areas{$area}++;
    
    $next = $previous = $areas{$area};
        
    $previous--;
    
    $next++;
    
    &buildmsg;
        
    &printhtml;		# output html
    
    }
}

############################################
# Build message
############################################

sub buildmsg {

	$MsgTemp = "\n";	# put a beginning CR in msg body.
	$kludges = "";

	#&showheader;

	foreach $s (@lines) 
	{
		$s =~ tr/\0-\037/\040-\100/;   
		$s =~ s/\s+$//;                
		$s=~tr/^\*/ /;
		$MsgTemp = $MsgTemp . "$s\n";
	}
    
	foreach $s (@kludges)
	{
		$kludges = $kludges . "$s\n";
	}    
}

############################################
# Print HTML format output
############################################

sub printhtml {

	$pagespec = getData( "HTML" );
	
	&addcontent;	
	
	unlink("$OUTDIR/$area/$areas{$area}.html");
	
	open(MSG,">$OUTDIR/$area/$areas{$area}.html") ||
	        &logging($logfile, $progid, ">$OUTDIR/$area/$areas{$area}.html",$!);
	binmode(MSG);
	
	print MSG "$pagespec";
	
	close(MSG);
	
	&updateindex();
	
	symlink ("./index.html", "$OUTDIR/$area/$next.html");
	
	undef @lines; 
	
}

############################################
# Replace placeholders.
############################################

sub addcontent {

    my ($pagetitle);	# local variable

#    $org = "1:$orgnet/$orgnode\n";	  # 11/29/01 rjc, commented out
#    $dest = "1:$destnet/$destnode\n"; # 11/29/01 rjc, commented out

    $pagetitle = sprintf "%s%spaceu", $area, ($next-1); 

    $pagespec =~ s/<!-- P2HVERS -->/$p2hvers/s;   # area name & msg #

    $pagespec =~ s/<!-- PAGETITLE -->/$pagetitle/s;   # area name & msg #

    $pagespec =~ s/<!-- AREA -->/$area/s;

    $pagespec =~ s/<!-- PREVIOUS -->/$previous/s;
    $pagespec =~ s/<!-- NEXT -->/$next/s;

    $pagespec =~ s/<!-- FROM -->/$from/s;
    $pagespec =~ s/<!-- TO -->/$to/s;
    $pagespec =~ s/<!-- SUBJECT -->/$subj/s;
    $pagespec =~ s/<!-- DATETIME -->/$datetime/s;
    $pagespec =~ s/<!-- MESSAGE -->/$MsgTemp/s;	
    $pagespec =~ s/<!-- MAILER -->/$mailer/s if ($mailer);
    $pagespec =~ s/<!-- ORIGIN -->/$origin/s;
    $pagespec =~ s/<!-- SEENBY -->/$seenby/s if ($seenby);
    $pagespec =~ s/<!-- KLUDGES -->/$kludges/s;
}

############################################
# Put the date in Human format
############################################

sub datetime {
#    local($sec, $min, $hour, $mday, $mon, $year, $wday);  need?
    @moy = ('jan','feb','mar','apr','may','jun',
            'jul','aug','sep','oct','nov','dec');
    @wdy = ('mon','tue','wed','thu','fri','say','sun');
    ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime();
    sprintf("%s, %d %s %d %02d:%02d:%02d",
            $wdy[$wday], $mday, $moy[$mon], $year, $hour, $min, $sec);
}

############################################
# Create index
############################################
# Needs $area
############################################

sub makenewindex {

	$index = getData( "HTMLINDEX" );
	
	$index =~ s/<!-- AREA -->/$area/s;
	
	open(INDEX,">$OUTDIR/$area/index.html");
	print INDEX "$index";
	close(INDEX);
	
	&updatemasterindex();
}

sub updateindex {

	$index = getData( "HTMLCONTENT" );	
	
	#$index =~ substr $index, 2;
	
	$index =~ s/<!-- FILE -->/$areas{$area}.html/s;
	$index =~ s/<!-- NUMBER -->/$areas{$area}/s;
	$index =~ s/<!-- FROM -->/$from/s;
	$index =~ s/<!-- TO -->/$to/s;
	$index =~ s/<!-- SUBJECT -->/$subj/s;
	$index =~ s/<!-- DATETIME -->/$datetime/s;

	open(INDEX,"$OUTDIR/$area/index.html");
	open(NEWINDEX, ">$OUTDIR/$area/index.tmp");

	while (<INDEX>)
	{
		
		if ( grep /^<!-- NEXT -->$/i, $_ )
		{
			chop $index;
			chomp $index;
			
			$_ =~ s/<!-- NEXT -->/$index/s;
			print NEWINDEX "$_";
		}
		else
		{
			print NEWINDEX "$_" if ( grep !/^\n\{1,\}/i, $_ );
		}
	}

	close(NEWINDEX);
	close(INDEX);
	
	copy("$OUTDIR/$area/index.tmp", "$OUTDIR/$area/index.html");
	unlink("$OUTDIR/$area/index.tmp");
}

sub updatemasterindex {

	opendir(DIR, $OUTDIR);
        @dirs = grep(!/.*\..*/i, readdir(DIR));
	closedir(DIR);
	
	$arealist = "";
	
	foreach $dir (sort @dirs) {
	
		#find description of echo
		
		foreach $echolist (@echolist)
		{
			
			$description = "No Description Available";
			
			$thisarea = $dir;
			$thisarea =~ tr/a-z/A-Z/;
			
			open LIST, "$echolist" or die "Unable to open $echolist: $!\n";
			
			while (<LIST>)
			{				
				chomp;
				
				if ( m/^$dir/i )
				{
					if ($DEBUG) { print "Description found\n" };
					
					($thisarea, $description) = split ' ', $_, 2;
					
					chomp ($description);
				}
			}
			
			close(LIST);
		}
		
		#get template
		$newline = getData ( "HTMLUPDATEMASTER" );
				
		#build line for list
		while ( $newline =~ m/<!-- AREA -->/ ) {
			$newline =~ s/<!-- AREA -->/$dir/s;
		}
		
		while ( $newline =~ m/<!-- NAME -->/ ) {
			$newline =~ s/<!-- NAME -->/$thisarea/s;
		}
		
		while ( $newline =~ m/<!-- DESCRIPTION -->/ ) {
			$newline =~ s/<!-- DESCRIPTION -->/$description/s;
		}
		
		#append line to list
		$arealist = "$arealist $newline";
	
	}
	
	#write new temporary index in $OUTDIR
	$index = getData( "HTMLNEWAREA" );
	
	$index =~ s/<!-- NEXT -->/$arealist/s;
	
	open(INDEX, ">$OUTDIR/index.tmp" );
	print INDEX $index;
	close(INDEX);
	
	#move temporary to live.
	copy("$OUTDIR/index.tmp", "$OUTDIR/index.html");
	unlink("$OUTDIR/index.tmp");

}

sub updatemasterindexold {

	if ( open(INDEX, "$OUTDIR/index.html") ) {
	       	close(INDEX);
	} else {
	       	$index = getData( "HTMLNEWAREA" );
				
		open(INDEX,">$OUTDIR/index.html");
		print INDEX "$index";
		close(INDEX);
        }

	$index = getData( "HTMLUPDATEMASTER" );	
	
	while ( $index =~ m/<!-- AREA -->/ ) {
		$index =~ s/<!-- AREA -->/$area/s;
	}
	
	open(INDEX,"$OUTDIR/index.html");
	open(NEWINDEX, ">$OUTDIR/index.tmp");
	
	while (<INDEX>)
	{
		if ( grep /^<!-- NEXT -->$/i, $_ )
		{
			chop $index;
			chomp $index;
			
			$_ =~ s/<!-- NEXT -->/$index/s;
			print NEWINDEX "$_";
		}
		else
		{
			print NEWINDEX "$_" if ( grep !/^\n\{1,\}/i, $_ );
		}
	}
	
	close(NEWINDEX);
	close(INDEX);
		
	copy("$OUTDIR/index.tmp", "$OUTDIR/index.html");
	unlink("$OUTDIR/index.tmp");
}

############################################
# Logging subroutine:  being replaced by
# in FTN::Utility
############################################

#sub log {

#    @moy = ('Jan','Feb','Mar','Apr','May','Jun',
#	        'Jul','Aug','Sep','Oct','Nov','Dec');
#    @time=localtime(time());

#    $progid="RPKT";

#    open (LOGFILE,">>$logfile") || die ("cannot open $logfile");
#    # print LOGFILE ("  $time[3] ",$moy[$time[4]]);
#    $time[3]<10 ? print LOGFILE ("  0$time[3] ",$moy[$time[4]]) : print LOGFILE ("  $time[3] ",$moy[$time[4]]);
#    if ($time[2])
#    {
#       $time[2]<10 ? print LOGFILE (" 0$time[2]:") : print LOGFILE ("$time[2]:");
#    }
#    else 
#    {
#       $time[2]<10 ? print LOGFILE (" 0$time[2]:") : print LOGFILE ("$time[2]:");
#    
#    }   
#    $time[1]<10 ? print LOGFILE ("0$time[1]:") : print LOGFILE ("$time[1]:");
#    $time[0]<10 ? print LOGFILE ("0$time[0] ","$progid") : print LOGFILE ("$time[0] ","$progid");
#    print LOGFILE (" @_\n");
#    close (LOGFILE);
#}

sub findhighmsg
{
	#Set $areas{$area} to high message.
	
	opendir(OUTDIR, "$OUTDIR/$area") or die "Unable to open $OUTDIR/$area: $!";
	
	my @countfiles = grep(/[0-9]\.*/i, readdir(OUTDIR));
	
	closedir(OUTDIR);
	
	if ( $countfiles[0] ) {
	
		$last = pop( @countfiles );
	
		@l = split (/\./, $last);
		
		$areas{$area} = $l[0];
		
		$areas{$area}--;
	}
}

# --------------------------------------------------------------------
# Function:  getData
# Returns:  section as a String, or undef if no section of the
#   provided name is found.
# Parameters:
#   $section    String with desired section name
# Description:  Allows for named sections of the __DATA__ block of this
#   script to be accessed.
# --------------------------------------------------------------------
{
my %data;
sub getData {
	( $section ) = @_;
	if ( ! %data ) {
		$block = '';
		while ( 1 ) {
			$line = <DATA>;
			if ( !defined( $line ) or $line =~ m/^\s*$/ ) {
				if ( $block ne '' ) {
					$block =~ s/\\\n\s*//gs;
					$block =~ s/^([A-Z_]+):(.*)$/$2/s;
					$data{$1} = $block;
					$block = ''
				}
				last if ( !defined( $line ) );
			} else {
				$block .= $line;
			}
		}
		close( DATA );
	}
	return ( defined( $data{$section} ) ? $data{$section} : undef );
}
}

sub cleanup {

    if ( -d $TEMPDIR ) {
	rmdir $TEMPDIR or die "Unable to remove $TEMPDIR: $!\n";
    }  # end if
}

#####################################################################
# DATA SECTION
#####################################################################
__DATA__
HTML:
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
<HTML>
<HEAD>
<META NAME="Generator" Content="pkt2html.pl <!-- P2HVERS -->">
<META HTTP-EQUIV="Content-Type" CONTENT="text/html;CHARSET=iso-8859-1">
<TITLE><!-- PAGETITLE --></TITLE>
</HEAD>
<body>
<center>
<table border="1" width="50%">
<tr>
	<td><a href="<!-- PREVIOUS -->.html">Previous Message</a></td>
	<td><a href="./index.html">Message List</a></td>
	<td><a href="../index.html">Area List</a></td>
	<td><a href="<!-- NEXT -->.html">Next Message</a></td>
<tr>
</table>
<p>
<table border="1" width="80%">
<tr>
	<td align="center"><!-- AREA --></td>
<tr>
	<td><B>From</B></td>
	<td><!-- FROM --></td>
</tr>
<tr>
	<td><B>To</B></td>
	<td><!-- TO --></td>
</tr>
<tr>
	<td><B>Subject</B></td>
	<td><!-- SUBJECT --></td>
</tr>
<tr>
	<td><B>Date</B></td>
	<td><!-- DATETIME --></td>
</tr>
</table>
<table border="1" width="80%">
<tr>
	<td>
		<pre>
		<!-- MESSAGE -->
		</pre>
	</td>
</tr>
<tr>
	<td>
		<center>
		<!-- MAILER -->
		Origin: <!-- ORIGIN -->
		<!-- <!-- SEENBY --> -->
		<!-- <!-- KLUDGES --> -->
		<P><!-- Standard Footer --></P>
		<h6 align="center">Copyright - 2002</h6>
		</center>
	</td>
</tr>
</table>
</center>
</body>
</HTML>

HTMLINDEX:
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
<HTML>
<HEAD>
  <META NAME="Generator" Content="pkt2html.pl <!-- P2HVERS -->">
  <META HTTP-EQUIV="Content-Type" CONTENT="text/html;CHARSET=iso-8859-1">
<TITLE><!-- PAGETITLE --></TITLE>
</HEAD>
<body>
<center>
<H1><!-- AREA --></H1><p>
<a href="../index.html">Area List</a><p>
<table border="1" width="90%">
<tr>
    <td><B>Message Number</B></td>
    <td><B>From</B></td>
    <td><B>To</B></td>
    <td><B>Subject</B></td>
    <td><B>Date</B></td>
</tr>
<!-- NEXT -->
</table>
</center>
<h6 align="center">Copyright - 2002</h6>
</center>
</body>
</HTML>

HTMLCONTENT:
<tr>
    <td align="center"><a href="<!-- FILE -->"><B><!-- NUMBER --></B></a></td>
    <td><!-- FROM --></td>
    <td><!-- TO --></td>
    <td><!-- SUBJECT --></td>
    <td><!-- DATETIME --></td>
</tr>
<!-- NEXT -->

HTMLNEWAREA:
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
<HTML>
<HEAD>
  <META NAME="Generator" Content="pkt2html.pl <!-- P2HVERS -->">
  <META HTTP-EQUIV="Content-Type" CONTENT="text/html;CHARSET=iso-8859-1">
  <TITLE><!-- PAGETITLE --></TITLE>
</HEAD>
<body>
<center>
<table border="1" width="90%">
<tr>
    <td><B>Message Area</B></td>
    <td><B>Description</B></td>
</tr>
<!-- NEXT -->
</table>
</center>
<h6 align="center">Copyright - 2002</h6>
</center>
</body>
</HTML>

HTMLUPDATEMASTER:
<tr>
    <td><a href="<!-- AREA -->/index.html"><B><!-- NAME --></B></a></td>
    <td><!-- DESCRIPTION --></td>
</tr>

